import contextlib
import json
import os
import time

import selenium.webdriver as webdriver
import selenium.webdriver.support.ui as ui
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from seleniumwire import webdriver

from timetoc.cli import TIME_TRACK_BASE_URL


def get_acess_token(email=None, password=None, headless=False):
    if email and password:
        chrome_options = Options()
        chrome_options.add_argument("--headless")
        drive = webdriver.Chrome(
            options=chrome_options, seleniumwire_options={"disable_encoding": True}
        )
    else:
        chrome_options = Options()
        if headless:
            chrome_options.add_argument("--headless")
        drive = webdriver.Chrome(
            seleniumwire_options={"disable_encoding": True}, options=chrome_options
        )

    with contextlib.closing(drive) as driver:
        driver.get(TIME_TRACK_BASE_URL)
        if email and password:
            wait_email = ui.WebDriverWait(driver, 60)  # timeout after 120 seconds
            wait_email.until(
                lambda driver: driver.find_element(By.CSS_SELECTOR, "#i0116")
            )
            email_input = driver.find_element(By.CSS_SELECTOR, "#i0116")
            email_input.send_keys(email)
            email_input.send_keys(Keys.ENTER)

            wait_password = ui.WebDriverWait(driver, 60)  # timeout after 120 seconds
            wait_password.until(
                lambda driver: driver.find_element(By.CSS_SELECTOR, "#i0118")
            )
            password_input = driver.find_element(By.CSS_SELECTOR, "#i0118")
            password_input.send_keys(password)
            password_input.send_keys(Keys.ENTER)
            password_input.send_keys(Keys.ENTER)

            time.sleep(2)
            sign_in = driver.find_element(By.CSS_SELECTOR, "#idSIButton9")
            sign_in.click()

            time.sleep(2)
            code = driver.find_element(By.CSS_SELECTOR, "#idRichContext_DisplaySign")
            print("Insert this code in the Authenticator App:", code.text)

        wait_login = ui.WebDriverWait(driver, 1000)  # timeout after 120 seconds
        wait_login.until(
            lambda driver: driver.find_element(By.ID, "panel-1010-innerCt")
        )
        for request in driver.requests:
            if request.response:
                if "load_all_ext" in request.url:
                    body = request.response.body
                    data = json.loads(str(body, "utf-8"))
                    access_token = data["userData"]["accessToken"]
                    token_file = open("token_file", "w")
                    token_file.write(access_token)
                    print("Access token obtained!")
                    return access_token



# from os import environ
import os
import tomllib
from datetime import datetime
from os import environ
from pathlib import Path

import typer
from rich.console import Console
from rich.prompt import Prompt

from timetoc.login import get_acess_token
from timetoc.timeparser import parse_date_range
from timetoc.timetracking import add_work_day

app = typer.Typer()
console = Console()

TIME_TRACK_BASE_URL = os.environ["TIME_TRACK_BASE_URL"]


@app.command()
def main():
    token_file = Path("./token_file")
    if token_file.is_file():
        access_token = open(token_file).readlines(0)[0]
    else:
        console.print("token file not found!", style="red")
        access_token = access_token_by_login()

    day_str = Prompt.ask("Day", default="yesterday")
    days = parse_date_range(day_str)
    for day in days:
        console.print("Day: ", day, style="bold red")
        start = Prompt.ask("Start Time", default="09:00")
        finish = Prompt.ask("Finish Time", default="17:00")
        break_start = Prompt.ask("Break Start", default="12:00")
        break_finish = Prompt.ask("Break Finish", default="13:00")
        is_home_office = typer.confirm("Is Home Office?", default=True)

        try:
            add_work_day(
                day=day,
                start=start,
                finish=finish,
                break_start=break_start,
                break_finish=break_finish,
                is_home_office=is_home_office,
                token=access_token,
            )
        except:
            console.print("Token expired please login again")
            access_token = access_token_by_login()
            add_work_day(
                day=day,
                start=start,
                finish=finish,
                break_start=break_start,
                break_finish=break_finish,
                is_home_office=is_home_office,
                token=access_token,
            )


def access_token_by_login():
    time_track_login = typer.confirm("Headless login?")
    if time_track_login:
        if not environ.get("TIME_TRACK_EMAIL") or environ.get("TIME_TRACK_PASSWORD"):
            email = Prompt.ask("Email")
            password = Prompt.ask("Password", password=True)
        else:
            email = environ["TIME_TRACK_EMAIL"]
            password = environ["TIME_TRACK_PASSWORD"]
        access_token = get_acess_token(email=email, password=password, headless=False)
    else:
        access_token = get_acess_token()

    return access_token


if __name__ == "__main__":
    app()



import datetime

import holidays
from dateutil.relativedelta import relativedelta


def parse_date_range(phrase, exclude_holidays=True, exclude_weekends=True):
    """
    Returns a list of datetime objects representing a date range based on a phrase describing the range.

    Parameters:
        phrase (str): A string describing the date range.
        exclude_holidays (bool): Whether to exclude Austrian public holidays from the resulting list of dates. Default is False.
        exclude_weekends (bool): Whether to exclude weekends (Saturdays and Sundays) from the resulting list of dates. Default is False.

    Returns:
        list: A list of datetime objects representing the dates in the range.
    """
    now = datetime.datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)

    if phrase == "today":
        start_date = now
        end_date = now + datetime.timedelta(days=1)
    elif phrase == "yesterday":
        start_date = now - datetime.timedelta(days=1)
        end_date = now
    elif phrase == "tomorrow":
        start_date = now + datetime.timedelta(days=1)
        end_date = start_date + datetime.timedelta(days=1)
    elif phrase == "this week":
        start_date = now - datetime.timedelta(days=now.weekday())
        end_date = start_date + datetime.timedelta(weeks=1)
    elif phrase == "last week":
        end_date = now - datetime.timedelta(days=now.weekday())
        start_date = end_date - datetime.timedelta(weeks=1)
    elif phrase == "next week":
        start_date = now + datetime.timedelta(days=(7 - now.weekday()))
        end_date = start_date + datetime.timedelta(weeks=1)
    elif phrase == "this month":
        start_date = now.replace(day=1)
        end_date = (start_date + relativedelta(months=1)).replace(day=1)
    elif phrase == "last month":
        end_date = now.replace(day=1)
        start_date = (end_date - relativedelta(months=1)).replace(day=1)

    date_range = []
    delta = datetime.timedelta(days=1)
    while start_date < end_date:
        if not exclude_weekends or start_date.weekday() not in [5, 6]:
            date_range.append(start_date)
        start_date += delta

    if exclude_holidays:
        at_holidays = holidays.AT(years=start_date.year)
        date_range = [
            d.strftime("%Y-%m-%d") for d in date_range if d not in at_holidays
        ]

    return date_range



import requests

from timetoc.login import TIME_TRACK_BASE_URL


def make_headers(token=None):
    return {
        "Authorization": f"Bearer {token}",
    }


def add_time_entry(
    day="2023-03-14",
    start_time="09:10",
    end_time="12:00",
    break_time=False,
    token=None,
):
    headers = make_headers(token)

    task_id = 4
    if break_time:
        task_id = 9

    data = {
        "0": {
            "user_id": 556,
            "task_id": task_id,
            "start_time_timezone": "Europe/Vienna",
            "end_time_timezone": "Europe/Vienna",
            "start_time": f"{day} {start_time}:00",
            "end_time": f"{day} {end_time}:00",
            "notes": "",
            "start_type_id": "0",
            "end_type_id": "0",
        },
    }

    response = requests.post(
        f"{TIME_TRACK_BASE_URL}/userapi/v3/timeTrackings/create/",
        headers=headers,
        json=data,
    )
    if response.status_code == 200:
        if break_time:
            print(
                f"✅ Break time entry starting at {start_time} ending at {end_time}  add successfully ☕ "
            )
            return

        print(
            f"✅ Time entry starting at {start_time} ending at {end_time}  add successfully 🕒"
        )


def add_home_office(day, token):
    headers = make_headers(token)
    data = {
        "user_id": "556",
        "subtype_id": "12",
        "type_id": "3",
        "from_date": f"{day}",
        "to_date": f"{day}",
        "request_duration_entity": "d",
        "duration": "1",
        "request_comment": "",
    }

    response_home_office = requests.post(
        f"{TIME_TRACK_BASE_URL}/userapi/v3/absences/create/",
        headers=headers,
        data=data,
    )
    if response_home_office.status_code == 200:
        print("✅ Home office entry add successfully 🏠")


def add_work_day(day, start, finish, break_start, break_finish, is_home_office, token):
    add_time_entry(day, start_time=start, end_time=break_start, token=token)
    add_time_entry(
        day, start_time=break_start, end_time=break_finish, break_time=True, token=token
    )
    add_time_entry(day, start_time=break_finish, end_time=finish, token=token)
    if is_home_office:
        add_home_office(day, token)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['timetoc']

package_data = \
{'': ['*']}

install_requires = \
['holidays>=0.21.13,<0.22.0',
 'pandas>=1.5.3,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.28.2,<3.0.0',
 'selenium-wire>=5.1.0,<6.0.0',
 'typer[all]>=0.7.0,<0.8.0',
 'webdriver-manager>=3.8.5,<4.0.0']

entry_points = \
{'console_scripts': ['timetoc = timetoc.cli:app']}

setup_kwargs = {
    'name': 'timetoc',
    'version': '0.1.0',
    'description': '',
    'long_description': '',
    'author': 'Guilherme Prokisch',
    'author_email': 'guilherme.prokisch@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
